# **************************************************************************
# *
# * Authors: Yunior C. Fonseca Reyna    (cfonseca@cnb.csic.es)
# *          Pablo Conesa Mingo         (pconesa@cnb.csic.es)
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

import unittest
#from metadataviewer.dao import *
#from metadataviewer.model import Table, Page


class TestStarFile(unittest.TestCase):

    def testCreatePageColumns(self):
        # pageFromStarFile = StarFile('datasets/output_particle.star')
        # table = Table('particles')
        # tableNames = pageFromStarFile.getTableNames()
        # self.assertEqual(len(tableNames), 2)
        # pageFromStarFile.fillTable(table)
        # self.assertEqual(len(table.getColumns()), 15)
        # page = Page(table, pageNumber=2, pageSize=10)
        # pageFromStarFile.fillPage(page, 0, True)
        # self.assertEqual(page.getSize(), 10)
        # pageFromStarFile.close()
        pass




