# **************************************************************************
# *
# * Authors: Yunior C. Fonseca Reyna    (cfonseca@cnb.csic.es)
# *
# *
# * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
# * 02111-1307  USA
# *
# *  All comments concerning this program package may be sent to the
# *  e-mail address 'scipion@cnb.csic.es'
# *
# **************************************************************************

#  -------ICONS----------

DOWN_ARROW = 'down-arrow.png'
UP_ARROW = 'up-arrow.png'
FOLDER = 'folder.png'
EXIT = 'exit.png'
TABLE = 'table.png'
TABLE_VIEW = 'table-view.png'
GALLERY_VIEW = 'gallery.png'
REDUCE_DECIMALS = 'reducedecimals.png'
INCREASE_DECIMALS = 'increasedecimals.png'
TABLE_BLOCKS = 'sections.png'
PREFERENCES = 'preferences.png'
ZOOM_PLUS = 'zoom.png'
GOTO_ITEM = 'goto_item.png'
HIDE = 'hide.png'
CLOSE = 'fa-close.png'
ERROR = 'error.png'
FROM_HERE = 'from_here.png'
TO_HERE = 'to_here.png'
PLUS = 'plus.png'
EXPORT_XLSX = 'export_xlsx.png'
EXPORT_CSV = 'export_csv.png'
OK = 'fa-ok.png'
PLOT = 'plot.png'

# -------LABELS PROPERTIES_TABLE ------------
LABEL = 'Label'
VISIBLE = 'Visible'
RENDER = 'Render'
EDIT = 'Edit'

# -------- LABELS PLOT TABLE -----------------
TYPE = 'Type'
PLOT_LABEL = 'Plot'
HISTOGRAM_LABEL = 'Histogram'
SCATTER_LABEL = 'Scatter'
BINS ='Bin'
XAXIS = 'X Axis'
LIMIT = 'Limit'
LIMIT_HELP = 'Limit of elements to be retrieved for use in the plot. \n ' \
             'If you uncheck the checkbox, the complete dataset is assumed.'
SELECTION = 'Plot selection ?'


# ------------ACTIONS TEXTS-------------------------

COLUMNS = 'Columns...'
PLOT_ACTION = 'Plot'
GO_TO_TABLE_VIEW = "Go to TABLE view"
GO_TO_GALLERY_VIEW = 'Go to GALLERY view'
REDUCE_DECIMALS_TEXT = 'Reduce decimals'
INCREASE_DECIMALS_TEXT = 'Increase decimals'
SORT_ASC = 'Sort ascending'
SORT_DESC = 'Sort descending'
GO_TO_ITEM = 'Go to item'

HIDE_COLUMN = 'Hide column'

SELECT = 'Select'
SELECT_ALL = 'All'
SELECT_FROM_HERE = 'From here'
SELECT_TO_HERE = 'To here'


# --------------TOOLBAR LABELS & VARIABLES----------------

BLOCKS = 'Tables'
ZOOM = 'Size in pixels'

# ---------------------CONSTANTS-----------------
ZOOM_SIZE = 250
ZOOM_MIN_SIZE = 50
ZOOM_MAX_SIZE = 2000
PAGE_SIZE = 50
DEFAULT_ROW_HEIGHT = 17
DEFAULT_STATUS_BAR_HEIGHT = 30
MAX_ITEMS_INDEX = 8000000
DEFAULT_BINS = 50
LIMIT_ROWS = 50000

# -------------------- STATUS BAR LABELS ------------------

CLOSE_BUTTON_TEXT = 'Close'
EXPORT_TO_CSV = 'Export to .csv'

# ------------------ DIALOG TEXTS ------------

CANCEL = 'Cancel'
ACCEPT = 'Accept'






